/*********************************************
* vim: set sw=8 ts=8 si :
* Author: Guido Socher, Copyright: GPL V2
* This is an electronic dice.
* See http://tuxgraphics.org/electronics/ 
* Chip type           : ATMEGA8
* Clock frequency     : Internal clock 1 Mhz (factory default) or 4 or 8 MHz 
*********************************************/
#include <avr/io.h>
#include <inttypes.h>
//
// decide which clock frequency you want (you need to set the fuse
// bits for this, see Makefile) and then select the right value here. F_CPU is
// needed for the delay loop.
#define F_CPU 1000000UL  // 1 MHz
//#define F_CPU 4000000UL  // 4 MHz
//#define F_CPU 8000000UL  // 8 MHz
//
#include <util/delay.h>


void delay_ms(unsigned int ms)
/* delay for a minimum of <ms> */
{
	// we use a calibrated macro. This is more
	// accurate and not so much compiler dependent
	// as self made code.
	while(ms){
		_delay_ms(0.96);
		ms--;
	}
}

void allLEDsOff(void)
{
	// + 4.5V, LEDs off
        PORTD |= (1<<PD0);
        PORTB |= (1<<PB6);
        PORTD |= (1<<PD7);
        PORTC |= (1<<PC4);
        PORTC |= (1<<PC2);
        PORTC |= (1<<PC0);
        PORTB |= (1<<PB2);
}

void allLEDsOn(void)
{
	// lamp test, all on:
        PORTD &= ~(1<<PD0);
        PORTB &= ~(1<<PB6);
        PORTD &= ~(1<<PD7);
        PORTC &= ~(1<<PC4);
        PORTC &= ~(1<<PC2);
        PORTC &= ~(1<<PC0);
        PORTB &= ~(1<<PB2);
}

// switch LEDs for a certain pattern on, num = 0 to 5
void displayNumber(unsigned char num)
{
	num++;
	if (num==1){
		PORTC &= ~(1<<PC0);
		return;
	}
	if (num==2){
		PORTD &= ~(1<<PD0);
		PORTB &= ~(1<<PB2);
		return;
	}
	if (num==3){
		PORTD &= ~(1<<PD0);
		PORTC &= ~(1<<PC0);
		PORTB &= ~(1<<PB2);
		return;
	}
	if (num==4){
		PORTD &= ~(1<<PD0);
		PORTD &= ~(1<<PD7);
		PORTC &= ~(1<<PC4);
		PORTB &= ~(1<<PB2);
		return;
	}
	if (num==5){
		PORTD &= ~(1<<PD0);
		PORTD &= ~(1<<PD7);
		PORTC &= ~(1<<PC4);
		PORTB &= ~(1<<PB2);
		PORTC &= ~(1<<PC0);
		return;
	}
	if (num==6){
		PORTD &= ~(1<<PD0);
		PORTB &= ~(1<<PB6);
		PORTD &= ~(1<<PD7);
		PORTC &= ~(1<<PC4);
		PORTC &= ~(1<<PC2);
		PORTB &= ~(1<<PB2);
		return;
	}
}

void initLEDports(void)
{
	// PD0, PB6, PD7, PC4, PC2, PC0, PB2
        // enable as output
        DDRD|= (1<<DDD0);
        DDRB|= (1<<DDB6);
        DDRD|= (1<<DDD7);
        DDRC|= (1<<DDC4);
        DDRC|= (1<<DDC2);
        DDRC|= (1<<DDC0);
        DDRB|= (1<<DDB2);
	// lamp test:
	allLEDsOn();
        delay_ms(500);
	allLEDsOff();
        delay_ms(200);
	//
	allLEDsOn();
        delay_ms(500);
	allLEDsOff();
}

int main(void)
{
	unsigned char i=0;
	unsigned char ledoff=0;
        /* Initialize */
	initLEDports();
	DDRB &= ~(1<<PINB0); // input line
	PORTB|= (1<<PINB0); // internal pullup resistor on, switch goes to ground

        while (1) {
		if (bit_is_clear(PINB,PINB0)){
			// switch off the LEDs once after button press:
			if (ledoff==0){
				allLEDsOff();
				ledoff=1;
			}
			i++;
			i %= 6; // numbers from 0 to 5
		}else{
			// no button press. display result:
			displayNumber(i);
			ledoff=0;
		}
        }
	return(0);
}

// end
